==> opengtl_0.9.12.orig.tar.bz2: OpenGTL-0.9.12/OpenShiva/OpenShiva/PixelConvertExpressionFactory_p.cpp <==
/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "PixelConvertExpressionFactory_p.h"

#include <llvm/Instructions.h>

#include "GTLCore/Type.h"
#include "GTLCore/AST/ConvertExpression.h"
#include "GTLCore/ExpressionResult_p.h"
#include "GTLCore/ExpressionGenerationContext_p.h"
#include "GTLCore/Macros_p.h"

#include "CodeGenerator_p.h"
#include "Debug.h"
#include "wrappers/PixelWrap_p.h"

using namespace OpenShiva;

namespace OpenShiva {
  class PixelConvertExpression : public GTLCore::AST::ConvertExpression {
    public:
      PixelConvertExpression( GTLCore::AST::Expression* _value,  const GTLCore::Type* _dstType ) : GTLCore::AST::ConvertExpression( _value ), m_dstType(_dstType)
      {
      }
      virtual ~PixelConvertExpression() {}
      virtual const GTLCore::Type* type() const { return m_dstType; }
      virtual GTLCore::ExpressionResult generateValue( GTLCore::GenerationContext& _gc, GTLCore::ExpressionGenerationContext& _egc ) const
      {
        if( value()->type()->dataType() == GTLCore::Type::STRUCTURE )
        {
          llvm::Value* pixelPtr = value()->generateValue( _gc, _egc).value();
        
          llvm::Value* pixelDataPtr = OpenShiva::CodeGenerator::accessPixelDataPtr( _gc, _egc.currentBasicBlock(), pixelPtr);
          return GTLCore::ExpressionResult( new llvm::LoadInst( pixelDataPtr, "", _egc.currentBasicBlock() ) , m_dstType );
        } else {
          return value()->generateValue( _gc, _egc);
        }
      }
    private:
      const GTLCore::Type* m_dstType;
  };
}

PixelConvertExpressionFactory::PixelConvertExpressionFactory()
{
}

PixelConvertExpressionFactory::~PixelConvertExpressionFactory()
{
}

GTLCore::AST::ConvertExpression* PixelConvertExpressionFactory::create( GTLCore::AST::Expression* value, const GTLCore::Type* _dstType ) const
{
  return new PixelConvertExpression( value, _dstType );
}

bool PixelConvertExpressionFactory::canConvertBetween( const GTLCore::Type* srcType, const GTLCore::Type* dstType) const
{
  return (srcType->dataType() == GTLCore::Type::STRUCTURE and srcType->structName().startWith( "pixel" ) and srcType->structDataMember(PixelWrap::INDEX_DATA).type() == dstType )
      or (dstType->dataType() == GTLCore::Type::STRUCTURE and dstType->structName().startWith( "pixel" ) and dstType->structDataMember(PixelWrap::INDEX_DATA).type() == srcType );
}

GTLCore::ConvertCenter::ConversionQuality PixelConvertExpressionFactory::conversionQuality( const GTLCore::Type* _srcType, const GTLCore::Type* _dstType ) const
{
  GTL_ASSERT( canConvertBetween( _srcType, _dstType) );
  UNUSED(_srcType);
  UNUSED(_dstType);
  return GTLCore::ConvertCenter::LOSSLESS_CONVERSION;
}

==> opengtl_0.9.14.orig.tar.bz2: OpenGTL-0.9.14/OpenShiva/OpenShiva/PixelConvertExpressionFactory_p.cpp <==
/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "PixelConvertExpressionFactory_p.h"

#include <llvm/Instructions.h>

#include "GTLCore/Type.h"
#include "GTLCore/AST/ConvertExpression.h"
#include "GTLCore/LLVMBackend/ExpressionResult_p.h"
#include "GTLCore/LLVMBackend/ExpressionGenerationContext_p.h"
#include "GTLCore/Macros_p.h"

#include "CodeGenerator_p.h"
#include "Debug.h"
#include "wrappers/PixelWrap_p.h"

using namespace OpenShiva;

namespace OpenShiva {
  class PixelConvertExpression : public GTLCore::AST::ConvertExpression {
    public:
      PixelConvertExpression( GTLCore::AST::Expression* _value,  const GTLCore::Type* _dstType ) : GTLCore::AST::ConvertExpression( _value ), m_dstType(_dstType)
      {
      }
      virtual ~PixelConvertExpression() {}
      virtual const GTLCore::Type* type() const { return m_dstType; }
      virtual LLVMBackend::ExpressionResult generateValue( LLVMBackend::GenerationContext& _gc, LLVMBackend::ExpressionGenerationContext& _egc ) const
      {
        if( value()->type()->dataType() == GTLCore::Type::STRUCTURE )
        {
          llvm::Value* pixelPtr = value()->generateValue( _gc, _egc).value();
        
          llvm::Value* pixelDataPtr = OpenShiva::CodeGenerator::accessPixelDataPtr( _gc, _egc.currentBasicBlock(), pixelPtr);
          return LLVMBackend::ExpressionResult( new llvm::LoadInst( pixelDataPtr, "", _egc.currentBasicBlock() ) , m_dstType );
        } else {
          return value()->generateValue( _gc, _egc);
        }
      }
      virtual GTLCore::AST::ExpressionResultSP generateValue( GTLCore::AST::GenerationVisitor* _generationVisitor) const
      {
        GTL_ABORT("UNIMPLEMENTED");
		return 0;
      }
    private:
      const GTLCore::Type* m_dstType;
  };
}

PixelConvertExpressionFactory::PixelConvertExpressionFactory()
{
}

PixelConvertExpressionFactory::~PixelConvertExpressionFactory()
{
}

GTLCore::AST::ConvertExpression* PixelConvertExpressionFactory::create( GTLCore::AST::Expression* value, const GTLCore::Type* _dstType ) const
{
  return new PixelConvertExpression( value, _dstType );
}

bool PixelConvertExpressionFactory::canConvertBetween( const GTLCore::Type* srcType, const GTLCore::Type* dstType) const
{
  return (srcType->dataType() == GTLCore::Type::STRUCTURE and srcType->structName().startWith( "pixel" ) and srcType->structDataMember(PixelWrap::INDEX_DATA).type() == dstType )
      or (dstType->dataType() == GTLCore::Type::STRUCTURE and dstType->structName().startWith( "pixel" ) and dstType->structDataMember(PixelWrap::INDEX_DATA).type() == srcType );
}

GTLCore::ConvertCenter::ConversionQuality PixelConvertExpressionFactory::conversionQuality( const GTLCore::Type* _srcType, const GTLCore::Type* _dstType ) const
{
  GTL_ASSERT( canConvertBetween( _srcType, _dstType) );
  UNUSED(_srcType);
  UNUSED(_dstType);
  return GTLCore::ConvertCenter::LOSSLESS_CONVERSION;
}
